@@ -4,10 +4,14 @@
 /* Author: Rainer Doemer			first version: 07/02/97 */
 /************************************************************************/
 
-/* last update: 05/26/01 */
+/* last update: 11/20/01 */
 
 /* modifications: (most recent first)
  *
+ * 11/20/01 RD	synchronized list of keywords with list in PRS_Scanner.l
+ * 11/20/01 RD	added support for 'new'/'delete' exceptions
+ *		(conditional compilation with GL_MEM_USE_EXCEPTIONS)
+ * 11/02/01 RD	moved IO support functions into separate file GL_FileIO.h
  * 05/26/01 RD	eliminated support for binary SIR files (import/export)
  * 05/25/01 RD	removed code not needed for the SCRC
  * 05/03/01 RD	replaced GL_OpenFileWithDirList() by GL_FindFileInDirList()
@@ -20,8 +24,6 @@
 #include <ctype.h>
 #include <assert.h>
 #include <errno.h>
-#include <sys/stat.h>
-#include <string.h>
 #include <unistd.h>
 #include <malloc.h>
 #include <stdarg.h>
@@ -77,8 +79,6 @@
 /*** internal variables *************************************************/
 
 
-static string	ErrorMessage;
-
 static void	*SpareMemory = NULL;
 static int	SpareMemorySize = 0;
 
@@ -87,6 +87,7 @@ static int	SpareMemorySize = 0;
 
 
 ERROR	GL_Error = GL_ERROR_NO_ERROR;	/* last error of this module */
+string	GL_ErrorMessage;
 
 int	GL_VerbosityLevel = 0;		/* (published by main module) */
 int	GL_WarningLevel = 0;		/* (published by main module) */
@@ -121,54 +122,76 @@ switch (ErrorNumber)
      case GL_ERROR_OUT_OF_MEMORY:
 	  return("Out of memory");
      case GL_ERROR_FILE_R_OPEN_FAILED:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "Cannot open file \"%s\" for read access"
 			+ GL_SystemErrorMessage */);
      case GL_ERROR_FILE_W_OPEN_FAILED:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "Cannot open file \"%s\" for write access"
 			+ GL_SystemErrorMessage */);
      case GL_ERROR_PROBLEM_WHILE_CLOSING_FILE:
 	  return("Problem while closing the file");
      case GL_ERROR_FILE_NOT_READABLE:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "File \"%s\" not readable"
 			+ GL_SystemErrorMessage */);
      case GL_ERROR_UNLINK_FAILED:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "Unlinking existing file \"%s\" failed"
 			+ GL_SystemErrorMessage */);
      case GL_ERROR_READ_BYTES_FAILED:
-	  return(ErrorMessage.chars()
-		/* "Reading %d bytes from file failed"
+	  return(GL_ErrorMessage.chars()
+		/* "Reading %d bytes from %s failed"
 			+ GL_SystemErrorMessage */);
      case GL_ERROR_WRITE_BYTES_FAILED:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "Writing %d bytes to file failed"
 			+ GL_SystemErrorMessage */);
      case GL_ERROR_NOT_AT_END_OF_FILE:
 	  return("Not at end of file: excess data found");
      case GL_ERROR_NOT_A_LEGAL_IDENTIFIER:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "'%s' is not a legal identifier" */);
      case GL_ERROR_NOT_A_LEGAL_IDENTIFIER_2:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "'%s' is not a legal identifier," GL_ERROR_MSG_NEWLINE
 		"it is a SpecC keyword" */);
      case GL_ERROR_NOT_A_LEGAL_IDENTIFIER_3:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "'%s' is not a legal identifier," GL_ERROR_MSG_NEWLINE
 		"it is reserved for future extensions" */);
      case GL_ERROR_NOT_A_LEGAL_FILENAME:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "'%s' is not a legal file name" */);
      case GL_ERROR_NOT_A_LEGAL_FILE_PATH:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "'%s' is not a legal file path" */);
+     case GL_ERROR_TMP_FILE_CREATE_FAILED:
+	  return(GL_ErrorMessage.chars()
+		/* "Temporary file creation failed"
+			+ GL_SystemErrorMessage */);
+     case GL_ERROR_TMP_FILE_REWIND_FAILED:
+	  return(GL_ErrorMessage.chars()
+		/* "Temporary file rewind failed"
+			+ GL_SystemErrorMessage */);
      case GL_ERROR_FILE_DELETE_FAILED:
-	  return(ErrorMessage.chars()
+	  return(GL_ErrorMessage.chars()
 		/* "Cannot delete file \"%s\""
 			+ GL_SystemErrorMessage */);
+     case GL_ERROR_ILLEGAL_ARGUMENTS:
+	  return("Illegal arguments (internal)");
+     case GL_ERROR_RW_MODE_VIOLATION:
+	  return("Read/write mode violation (internal)");
+     case GL_ERROR_FILE_RW_OPEN_FAILED:
+	  return(GL_ErrorMessage.chars()
+		/* "Cannot open file \"%s\" for read/write access"
+			+ GL_SystemErrorMessage */);
+     case GL_ERROR_FILE_IO_ERROR:
+	  return(GL_ErrorMessage.chars()
+		/* "File I/O error"
+			+ GL_SystemErrorMessage */);
+     case GL_ERROR_NOT_AT_END_OF_STRING:
+	  return("Not at end of string: excess data found");
     } /* hctiws */
 
 return("Unknown error number (internal)");
@@ -308,6 +331,9 @@ va_end(ArgPtr);
 	/* overload C++ memory management */
 
 void *operator new (size_t NumBytes)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw (std::bad_alloc)
+#endif /* GL_MEM_USE_EXCEPTIONS */
 {
 void	*MemAddress;
 
@@ -327,6 +353,9 @@ return(MemAddress);
 
 
 void *operator new[] (size_t NumBytes)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw (std::bad_alloc)
+#endif /* GL_MEM_USE_EXCEPTIONS */
 {
 void	*MemAddress;
 
@@ -346,6 +375,9 @@ return(MemAddress);
 
 
 void operator delete (void *MemAddress)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw ( )
+#endif /* GL_MEM_USE_EXCEPTIONS */
 {
 
 #ifdef MEM_DEBUG
@@ -360,6 +392,9 @@ if (MemAddress)		/* delete must tolerate NULL argument */
 
 
 void operator delete[] (void *MemAddress)
+#ifdef GL_MEM_USE_EXCEPTIONS
+	throw ( )
+#endif /* GL_MEM_USE_EXCEPTIONS */
 {
 
 #ifdef MEM_DEBUG
@@ -479,12 +514,12 @@ ERROR GL_CheckIdentifier(		/* checks for a legal identifier */
 const char	*CharPtr;
 
 if (Name == NULL)
-   { ErrorMessage = "'<NULL>' is not a legal identifier";
+   { GL_ErrorMessage = "'<NULL>' is not a legal identifier";
      return(GL_ERROR_NOT_A_LEGAL_IDENTIFIER);
     } /* fi */
 if (  (!(isalpha(*Name)))
     &&(*Name != '_'))
-   { ErrorMessage.form("'%s' is not a legal identifier", Name);
+   { GL_ErrorMessage.form("'%s' is not a legal identifier", Name);
      return(GL_ERROR_NOT_A_LEGAL_IDENTIFIER);
     } /* fi */
 
@@ -492,7 +527,7 @@ CharPtr = Name + 1;
 while(*CharPtr)
    { if (  (!(isalnum(*CharPtr)))
 	 &&(*CharPtr != '_'))
-	{ ErrorMessage.form("'%s' is not a legal identifier", Name);
+	{ GL_ErrorMessage.form("'%s' is not a legal identifier", Name);
 	  return(GL_ERROR_NOT_A_LEGAL_IDENTIFIER);
 	 } /* fi */
      CharPtr++;
@@ -506,6 +541,9 @@ return(GL_ERROR_NO_ERROR);
 ERROR GL_CheckSpecCIdentifier(		/* checks for a legal identifier */
 	const char	*Name)		/* and not being a SpecC keyword */
 {
+
+// note: the following lists should match the ones in PRS_Scanner.l!
+
 static const char *Keyword[] = {
 	"auto",
 	"break",
@@ -570,9 +608,14 @@ static const char *Keyword[] = {
 	"waitfor",
 	NULL };
 static const char *Reserved[] = {
+	"and",
+	"and_eq",
 	"asm",
+	"bitand",
+	"bitor",
 	"catch",
 	"class",
+	"compl",
 	"const_cast",
 	"delete",
 	"dynamic_cast",
@@ -583,7 +626,11 @@ static const char *Reserved[] = {
 	"mutable",
 	"namespace",
 	"new",
+	"not",
+	"not_eq",
 	"operator",
+	"or",
+	"or_eq",
 	"private",
 	"protected",
 	"public",
@@ -595,6 +642,9 @@ static const char *Reserved[] = {
 	"typename",
 	"using",
 	"virtual",
+	"wchar_t",
+	"xor",
+	"xor_eq",
 	NULL };
 int		Index;
 
@@ -605,7 +655,7 @@ if ((GL_Error = GL_CheckIdentifier(Name)))
 Index = 0;		/* there's surely a more efficient way to do this, */
 while(Keyword[Index])	/* but this function usually is not time-critical  */
    { if (0 == strcmp(Keyword[Index], Name))
-	{ ErrorMessage.form(
+	{ GL_ErrorMessage.form(
 		"'%s' is not a legal identifier," GL_ERROR_MSG_NEWLINE
 		"it is a SpecC keyword", Name);
 	  return(GL_ERROR_NOT_A_LEGAL_IDENTIFIER_2);
@@ -616,7 +666,7 @@ while(Keyword[Index])	/* but this function usually is not time-critical  */
 Index = 0;
 while(Reserved[Index])
    { if (0 == strcmp(Reserved[Index], Name))
-	{ ErrorMessage.form(
+	{ GL_ErrorMessage.form(
 		"'%s' is not a legal identifier," GL_ERROR_MSG_NEWLINE
 		"it is reserved for future extensions", Name);
 	  return(GL_ERROR_NOT_A_LEGAL_IDENTIFIER_3);
@@ -635,7 +685,7 @@ ERROR GL_CheckFilename(		/* checks for a legal filename */
 const char	*CharPtr;
 
 if (Filename == NULL)
-   { ErrorMessage = "'<NULL>' is not a legal file name";
+   { GL_ErrorMessage = "'<NULL>' is not a legal file name";
      return(GL_ERROR_NOT_A_LEGAL_FILENAME);
     } /* fi */
 
@@ -644,7 +694,7 @@ while(*CharPtr)
    { if (  (isspace(*CharPtr))
 	 ||(iscntrl(*CharPtr))
 	 ||(*CharPtr == '/'))
-	{ ErrorMessage.form("'%s' is not a legal file name", Filename);
+	{ GL_ErrorMessage.form("'%s' is not a legal file name", Filename);
 	  return(GL_ERROR_NOT_A_LEGAL_FILENAME);
 	 } /* fi */
      CharPtr++;
@@ -661,7 +711,7 @@ ERROR GL_CheckPathFilename(	/* checks for a legal filename */
 const char	*CharPtr;
 
 if (PathFile == NULL)
-   { ErrorMessage = "'<NULL>' is not a legal file path";
+   { GL_ErrorMessage = "'<NULL>' is not a legal file path";
      return(GL_ERROR_NOT_A_LEGAL_FILE_PATH);
     } /* fi */
 
@@ -669,7 +719,7 @@ CharPtr = PathFile;
 while(*CharPtr)
    { if (  (isspace(*CharPtr))
 	 ||(iscntrl(*CharPtr)))
-	{ ErrorMessage.form("'%s' is not a legal file path", PathFile);
+	{ GL_ErrorMessage.form("'%s' is not a legal file path", PathFile);
 	  return(GL_ERROR_NOT_A_LEGAL_FILE_PATH);
 	 } /* fi */
      CharPtr++;
@@ -725,237 +775,6 @@ return(BaseName.chars());
 } /* end of GL_GetBaseName */
 
 
-/*** file I/O ***********************************************************/
-
-
-	/* support functions for file I/O */
-
-FILE *GL_OpenFileForReading(	/* open a file for read access */
-	const char *Filename)	/* (supports "-" for stdin) */
-{
-FILE	*FileHandle;
-
-assert(Filename != NULL);
-
-if (0 == strcmp(Filename, "-"))
-   { return(stdin);
-    } /* fi */
-
-if ((GL_Error = GL_FileIsReadable(Filename)))
-   { return(NULL);
-    } /* fi */
-
-if (!(FileHandle = fopen(Filename, READ_ACCESS_CODE)))
-   { ErrorMessage.form("Cannot open file \"%s\" for read access",
-			Filename);
-     ErrorMessage += GL_SystemErrorMessage();
-     GL_Error = GL_ERROR_FILE_R_OPEN_FAILED;
-     return(NULL);
-    } /* fi */
-
-return(FileHandle);
-
-} /* end of GL_OpenFileForReading */
-
-
-FILE *GL_OpenFileForWriting(	/* open a (new) file for write access */
-	const char *Filename)	/* (supports "-" for stdout) */
-{
-FILE	*FileHandle;
-
-if (0 == strcmp(Filename, "-"))
-   { return(stdout);
-    } /* fi */
-
-if ((GL_Error = GL_FileIsCreatable(Filename)))
-   { return(NULL);
-    } /* fi */
-
-if (!(FileHandle = fopen(Filename, WRITE_ACCESS_CODE)))
-   { ErrorMessage.form("Cannot open file \"%s\" for write access",
-			Filename);
-     ErrorMessage += GL_SystemErrorMessage();
-     GL_Error = GL_ERROR_FILE_W_OPEN_FAILED;
-     return(NULL);
-    } /* fi */
-
-return(FileHandle);
-
-} /* end of GL_OpenFileForWriting */
-
-
-void GL_CloseFile(		/* close a file */
-	FILE *FileHandle)
-{
-
-assert(FileHandle != NULL);
-
-if (  (FileHandle != stdin)
-    &&(FileHandle != stdout))
-   { fclose(FileHandle);
-    } /* fi */
-
-} /* end of GL_CloseFile */
-
-
-ERROR GL_CloseFileWithCheck(	/* close a file and check for errors */
-	FILE *FileHandle)
-{
-
-assert(FileHandle != NULL);
-
-if (ferror(FileHandle))
-   { if (  (FileHandle != stdin)
-	 &&(FileHandle != stdout))
-	{ fclose(FileHandle);
-	 } /* fi */
-     return(GL_ERROR_PROBLEM_WHILE_CLOSING_FILE);
-    } /* fi */
-
-if (  (FileHandle != stdin)
-    &&(FileHandle != stdout))
-   { if (fclose(FileHandle))
-	{ return(GL_ERROR_PROBLEM_WHILE_CLOSING_FILE);
-	 } /* fi */
-    } /* fi */
-
-return(GL_ERROR_NO_ERROR);
-
-} /* end of GL_CloseFileWithCheck */
-
-
-ERROR GL_DeleteFile(		/* delete a file */
-	const char	*Filename)
-{
-if (unlink(Filename))
-   { ErrorMessage.form("Cannot delete file \"%s\"",
-			Filename);
-     ErrorMessage += GL_SystemErrorMessage();
-     return(GL_ERROR_FILE_DELETE_FAILED);
-    } /* fi */
-
-return(GL_ERROR_NO_ERROR);
-
-} /* end of GL_DeleteFile */
-
-
-ERROR GL_FileIsReadable(	/* check if file is read-accessable */
-	const char	*Filename)	/* (supports "-" for stdin) */
-{
-struct stat	StatBuf;
-
-assert(Filename != NULL);
-
-if (0 == strcmp(Filename, "-"))
-   { return(GL_ERROR_NO_ERROR);
-    } /* fi */
-
-if (0 != access(Filename, R_OK))
-   { ErrorMessage.form("File \"%s\" not readable%s",
-			Filename, GL_SystemErrorMessage());
-     return(GL_ERROR_FILE_NOT_READABLE);
-    } /* fi */
-if (0 != stat(Filename, &StatBuf))
-   { ErrorMessage.form("File \"%s\" not readable%s",
-			Filename, GL_SystemErrorMessage());
-     return(GL_ERROR_FILE_NOT_READABLE);
-    } /* fi */
-if ((StatBuf.st_mode & S_IFMT) != S_IFREG)
-   { ErrorMessage.form("File \"%s\" not readable;" GL_ERROR_MSG_NEWLINE
-			"not an ordinary file",
-			Filename);
-     return(GL_ERROR_FILE_NOT_READABLE);
-    } /* fi */
-
-return(GL_ERROR_NO_ERROR);
-
-} /* end of GL_FileIsReadable */
-
-
-ERROR GL_FileIsCreatable(	/* check if file is write-accessable */
-	const char	*Filename)	/* (supports "-" for stdout) */
-{
-
-assert(Filename != NULL);
-
-if (0 == strcmp(Filename, "-"))
-   { return(GL_ERROR_NO_ERROR);
-    } /* fi */
-
-if (0 == access(Filename, F_OK))	/* existing? */
-   { if (0 == unlink(Filename))
-	{ return(GL_ERROR_NO_ERROR);	/* removed! */
-	 } /* fi */
-     else
-	{ ErrorMessage.form("Unlinking existing file \"%s\" failed%s",
-			Filename, GL_SystemErrorMessage());
-	  return(GL_ERROR_UNLINK_FAILED);
-	 } /* fi */
-    } /* fi */
-
-return(GL_ERROR_NO_ERROR);
-
-} /* end of GL_FileIsCreatable */
-
-
-ERROR GL_ReadEOF(		/* read (and check) EOF from file */
-	FILE		*FileHandle)
-{
-char	Trash;
-
-fread(&Trash, 1, 1, FileHandle);	/* try to read one more byte */
-if (!(feof(FileHandle)))
-   { clearerr(FileHandle);
-     return(GL_ERROR_NOT_AT_END_OF_FILE);
-    } /* fi */
-
-clearerr(FileHandle);
-
-return(GL_ERROR_NO_ERROR);
-
-} /* end of GL_ReadEOF */
-
-
-const char *GL_FindFileInDirList(	/* find a file for read access */
-	const char	*Filename,	/* (applying DirList and Suffix) */
-	const char	*Suffix = NULL,	/* (returns NULL if not found) */
-	GL_STRINGLIST	*DirList = NULL)
-{
-GL_STRINGLIST	*Dir;
-static string	NewFilename;	/* return buffer! */
-
-assert(Filename != NULL);
-
-if (Filename[0] == '/')		/* absolute filename? */
-   { NewFilename = Filename;
-     if (Suffix)
-	{ NewFilename += Suffix;
-	 } /* fi */
-     if ((GL_ERROR_NO_ERROR == GL_FileIsReadable(NewFilename.chars())))
-	{ return(NewFilename.chars());
-	 } /* fi */
-     return(NULL);	/* absolute file not found */
-    } /* fi */
-
-Dir = DirList;		/* apply the list of directories */
-while(Dir)
-   { NewFilename = Dir->String;
-     NewFilename += "/";
-     NewFilename += Filename;
-     if (Suffix)
-	{ NewFilename += Suffix;
-	 } /* fi */
-     if ((GL_ERROR_NO_ERROR == GL_FileIsReadable(NewFilename.chars())))
-	{ return(NewFilename.chars());
-	 } /* fi */
-     Dir = Dir->Succ;
-    } /* elihw */
-
-return(NULL);	/* no such file found in directory list */
-
-} /* end of GL_FindFileInDirList */
-
-
 /************************************************************************/
 /*** main (for debugging only)					      ***/
 /************************************************************************/
