@@ -4,10 +4,13 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 05/31/01 */
+/* last update: 11/29/01 */
 
 /* modifications: (most recent first)
  *
+ * 11/29/01 RD	provided isascii() macro for all platforms
+ * 11/21/01 RD	took out default arguments from function definitions
+ * 08/13/01 RD	switched type of bitvector length to unsigned
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
  * 05/05/01 RD	print bitvector constants as template types
@@ -80,6 +83,10 @@
 #define SUPPORT_ISO
 #endif
 
+#ifndef isascii			/* isascii() not available on some platforms */
+#define isascii(i) ((unsigned char)(i) <= 127)
+#endif /* isascii */
+
 
 /*** internal type declarations *****************************************/
 
@@ -124,8 +131,8 @@ static void SIR_ConvertLD(
 SIR_Constant::SIR_Constant(		/* constructor #1 */
 	SIR_CONSTTYPE	Type,
 	int		Value,
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -171,8 +178,8 @@ switch(Type)
 SIR_Constant::SIR_Constant(		/* constructor #2 */
 	SIR_CONSTTYPE	Type,
 	LONG_LONG	Value,
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -243,8 +250,8 @@ switch(Type)
 SIR_Constant::SIR_Constant(		/* constructor #3 */
 	SIR_CONSTTYPE	Type,
 	UNSIGNED_LONG_LONG	Value,
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -315,8 +322,8 @@ switch(Type)
 SIR_Constant::SIR_Constant(		/* constructor #4 */
 	SIR_CONSTTYPE	Type,
 	long double	Value,
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -346,8 +353,8 @@ switch(Type)
 SIR_Constant::SIR_Constant(		/* constructor #5 */
 	SIR_CONSTTYPE	Type,
 	string		*Value,	/* (the string is consumed!) */
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -369,8 +376,8 @@ switch(Type)
 SIR_Constant::SIR_Constant(		/* constructor #6 */
 	SIR_CONSTTYPE	Type,
 	const char	*Value,	/* (the string is copied!) */
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -399,8 +406,8 @@ switch(Type)
 
 SIR_Constant::SIR_Constant(		/* constructor #7 (for bitvectors) */
 	sir_bit		Value,		/* (the bitvector is copied) */
-	unsigned int	Line = 0,
-	sir_fileinfo	*FileInfo = NULL) :
+	unsigned int	Line /* = 0 */,
+	sir_fileinfo	*FileInfo /* = NULL */) :
 		SIR_Node(Line, FileInfo)
 {
 
@@ -584,7 +591,7 @@ return(this);	/* return myself after promotion */
 
 sir_constant *SIR_Constant::Converted(	/* converted constant */
 	SIR_CONSTTYPE	Type,		/* (supports all legal conversions) */
-	int		Length = 0)	/* for bitvecs; 0 is natural length */
+	unsigned int	Length /* = 0 */) /* for bitvecs; 0 is natural length */
 {
 sir_bit		*BitVec;
 
@@ -882,7 +889,7 @@ while(*CharPtr)
 
 
 const char *SIR_Constant::Print(	/* return text string of value */
-	BOOL		CppNotation = FALSE)	/* default: SpecC notation */
+	BOOL		CppNotation /* = FALSE */) /* default: SpecC notation */
 {
 static string	PrintedValue;
 char		NumberBuffer[MAX_NUM_LENGTH];
@@ -1133,7 +1140,6 @@ return(PrintedValue.chars());
 } /* end of SIR_Constant::Print */
 
 
-
 /*** internal functions *************************************************/
 
 
