@@ -4,10 +4,11 @@
 /* Author: Rainer Doemer			first version: 10/10/97 */
 /************************************************************************/
 
-/* last update: 05/31/01 */
+/* last update: 11/08/01 */
 
 /* modifications: (most recent first)
  *
+ * 11/08/01 RD	removed SIR_ERROR_WRITING_SPECC_FILE
  * 05/31/01 RD	removed code not needed for the SCRC
  * 02/14/01 RD	started this list (last changes was 04/08/99)
  */
@@ -210,9 +211,6 @@ switch (ErrorNumber)
      case SIR_ERROR_REDEFINITION_OF_INTERFACE_2:
 	  return(SIR_ErrMsg.chars()
 	/* "Redefinition of %s '%s'" */);
-     case SIR_ERROR_WRITING_SPECC_FILE:
-	  return(SIR_ErrMsg.chars()
-	/* "Writing to file failed" + GL_SystemErrorMessage() */);
      case SIR_ERROR_DOUBLE_IMP_SYMBOL_NOT_FOUND:
 	  return("Error while importing:" GL_ERROR_MSG_NEWLINE
 		"double-imported symbol not found (internal)");
