@@ -4,10 +4,11 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 05/29/01 */
+/* last update: 11/08/01 */
 
 /* modifications: (most recent first)
  *
+ * 11/08/01 RD	switched code generation to use GL_IO layer
  * 05/29/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
  * 04/19/01 RD	fixed a bug in test 1 (TypeBitVec7)
@@ -702,7 +703,7 @@ sir_constant	*MyConst;
 sir_expression	*MyExpression;
 sir_expressions	*MyArgs;
 sir_label	*MyLabel;
-FILE		*FileHandle;
+GL_HANDLE	*IO;
 ERROR		Error;
 
 
@@ -961,23 +962,22 @@ puts(" Done.");
 
 printf("\tGenerate SpecC code of the design... (\"HelloWorld2.sc\")");
 
-if (!(FileHandle = GL_OpenFileForWriting("HelloWorld2.sc")))
+if (!(IO = GL_IO::OpenFile("HelloWorld2.sc", GL_IO_WRITE)))
    { GL_PrintError(GL_Error, GL_ErrorText(GL_Error));
      puts("Opening file 'HelloWorld2.sc' failed, test aborted!");
-     GL_CloseFile(FileHandle);
      delete Design1;
      return(RESULT_ERROR);
     } /* fi */
 
-if ((Error = Design1->WriteSC(FileHandle, "HelloWorld2.sc")))
+if ((Error = Design1->WriteSC(IO, "HelloWorld2.sc")))
    { GL_PrintError(Error, SIR_ErrorText(Error));
      puts("Source code generation failed, test aborted!");
-     GL_CloseFile(FileHandle);
+     IO->Close();
      delete Design1;
      return(RESULT_ERROR);
     } /* fi */
 
-if ((Error = GL_CloseFileWithCheck(FileHandle)))
+if ((Error = IO->CloseWithCheck()))
    { GL_PrintError(Error, GL_ErrorText(Error));
      puts("Writing design failed, test aborted!");
      delete Design1;
