@@ -4,10 +4,17 @@
 /* Author: Rainer Doemer			first version: 10/08/97 */
 /************************************************************************/
 
-/* last update: 05/31/01 */
+/* last update: 11/21/01 */
 
 /* modifications: (most recent first)
  *
+ * 11/21/01 RD	took out unneeded class qualification for method WriteIntro
+ * 11/08/01 RD	switched code generation to use GL_IO layer
+ * 10/18/01 RD	SpecC code generator optimizes away redundant method
+ *		declarations (added SIR_Symbol::IsRedundantMethodDecl())
+ * 10/16/01 RD	implemented code generation by order of source line info
+ * 10/11/01 RD	improved overall formatting of generated code and
+ *		enabled section comments in the code
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
  * 05/25/01 RD	eliminated support for automatic IP wrapping (scc -ip option)
@@ -28,7 +35,12 @@
 /*** constants and macros ***********************************************/
 
 
-// #define SIR_PUT_COMMENTS_IN_SOURCE	// put comments in generated source code
+#define SIR_PUT_COMMENTS_IN_SOURCE	// put comments in generated code
+
+#define SIR_SORT_CODE_BY_LINEINFO	// sort generated code by line info
+					// (otherwise by alphabetical order)
+
+#define SIR_OPTIMIZE_METHOD_DECLS	// eliminate redundant method decls.
 
 
 /*** enumeration types **************************************************/
@@ -242,6 +254,9 @@ BOOL Instantiates(		/* checks for instantiation */
 BOOL Contains(			/* checks for containment */
 	sir_symbol	*ClassSymbol);	/* (behaviors and channels only) */
 
+BOOL IsRedundantMethodDecl(	/* checks if method needs a declaration */
+	sir_symbol_ptrs	*DefList);
+
 
 sir_symbols *GetScope(void);	/* determines the scope of this symbol */
 				/* (returns NULL if not in a table) */
@@ -318,42 +333,40 @@ static const char *PrintStorageClass(	/* prints SpecC storage class #2 */
 	int		PipeStages = 0,
 	BOOL		AppendSpace = false);
 
-
 ERROR WriteSC(			/* (re-) generates SpecC source code */
-	FILE		*File,	/* (for non-class symbols) */
+	GL_HANDLE	*IO,	/* (for non-class symbols) */
 	BOOL		WriteNotes,
 	BOOL		AsDeclaration = FALSE,
 	BOOL		CplusplusMode = FALSE,
 	BOOL		AsReference = FALSE);
 
 ERROR WriteSC2(			/* (re-) generates SpecC source code */
-	FILE		*File,	/* (for class symbols) */
+	GL_HANDLE	*IO,	/* (for class symbols) */
 	BOOL		WriteNotes,
 	BOOL		AsDeclaration = FALSE);
 
 ERROR WriteSC3(			/* (re-) generates SpecC source code */
-	FILE		*File,	/* (for function definitions) */
+	GL_HANDLE	*IO,	/* (for function definitions) */
 	BOOL		WriteNotes,
 	BOOL		CplusplusMode = FALSE);
 
 ERROR WriteCC2(			/* generates C++ source code */
-	FILE		*File,	/* (for class symbols) */
+	GL_HANDLE	*IO,	/* (for class symbols) */
 	BOOL		WriteNotes,
 	BOOL		AsDeclaration = FALSE);
 
 ERROR WriteCC2b(		/* generates C++ source code */
-	FILE		*File,	/* (for classes, method implementations) */
+	GL_HANDLE	*IO,	/* (for classes, method implementations) */
 	BOOL		WriteNotes);
 
 ERROR WriteCC2IP(		/* generates C++ source code */
-	FILE		*File);	/* (for IP classes) */
+	GL_HANDLE	*IO);	/* (for IP classes) */
 
 ERROR WriteSC4ClassBody(	/* generates SpecC code for class bodies */
-	FILE		*File,	/* (for class definitions only) */
+	GL_HANDLE	*IO,	/* (for class definitions only) */
 	BOOL		WriteNotes = TRUE,
 	BOOL		WriteLines = TRUE,
-	const char	*ThisFilename = NULL,	/* default location if */
-	unsigned int	ThisLineNumber = 1);	/* line info unknown   */
+	const char	*ThisFilename = NULL);	/* default location */
 
 
 ERROR CheckInitializer(		/* checks whether initializer is legal */
@@ -496,31 +509,48 @@ ERROR DefineClass(		/* makes class declaration a definition */
 
 
 ERROR WriteSC(			/* (re-) generates SpecC source code */
-	FILE		*File,
+	GL_HANDLE	*IO,
 	BOOL		WriteNotes);
 
 ERROR WriteOrderedSC1(		/* writes ordered class instantiations */
 	sir_symbol	*Current,	/* (recursively) */
-	FILE		*File,
-	BOOL		WriteNotes);
+	GL_HANDLE	*IO,
+	BOOL		WriteNotes,
+	BOOL		*PutSeparator,
+	BOOL		*PutIntro,
+	BOOL		PutComments);
 
 ERROR WriteOrderedSC2(		/* writes ordered class definitions */
 	sir_symbol	*Current,	/* (recursively) */
-	FILE		*File,
-	BOOL		WriteNotes);
+	GL_HANDLE	*IO,
+	BOOL		WriteNotes,
+	BOOL		*PutSeparator,
+	BOOL		*PutIntro,
+	BOOL		PutComments);
 
 ERROR WriteH(			/* generates C++ header code */
-	FILE		*File,
+	GL_HANDLE	*IO,
 	BOOL		WriteNotes);
 
 ERROR WriteCC(			/* generates C++ main code */
-	FILE		*File,
+	GL_HANDLE	*IO,
 	BOOL		WriteNotes);
 
 ERROR WriteOrderedCC2(		/* writes ordered C++ class definitions */
 	sir_symbol	*Current,	/* (recursively) */
-	FILE		*File,
-	BOOL		WriteNotes);
+	GL_HANDLE	*IO,
+	BOOL		WriteNotes,
+	BOOL		*PutSeparator,
+	BOOL		*PutIntro,
+	BOOL		PutComments);
+
+static void WriteIntro(		/* create spacing, etc. before items */
+	GL_HANDLE	*IO,
+	bool		SeparateItems,
+	bool		*PutIntro,
+	bool		*PutSeparator,
+	bool		PutComment,
+	const char	*Comment);
 };
 
 
@@ -577,6 +607,11 @@ void UnAlias(void);		/* unalias all type, usertype, symbol links */
 int CmpSymbolNames(		/* compare all symbol names */
 	sir_symbol_ptrs	*Others);	/* (similar to strcmp) */
 
+
+void InsertByLineInfo(		/* sort symbols by source line information */
+	sir_symbol_ptr	*SymbolPtr);
+
+
 sir_symbol_ptr *Find(		/* searches for a specific entry */
 	sir_symbol	*Symbol);	/* (returns NULL if not found) */
 };
