@@ -4,10 +4,14 @@
 /* Author: Rainer Doemer			first version: 10/07/97 */
 /************************************************************************/
 
-/* last update: 05/31/01 */
+/* last update: 11/29/01 */
 
 /* modifications: (most recent first)
  *
+ * 11/29/01 RD	inserted a missing 'const' qualifier
+ * 11/21/01 RD	took out default arguments from function definitions
+ * 11/01/01 RD	eliminated trailing space when printing abstract types
+ * 10/31/01 RD	added SIR_Type::PrettyString() with separated pre/post-fix
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
  * 05/05/01 RD	changed use of 'long long' to 'LONG_LONG' (forgotten case)
@@ -53,7 +57,7 @@
 /*** internal variables *************************************************/
 
 
-static char	*TypeName[] =	/* table of built-in type names */
+static const char *TypeName[] =	/* table of built-in type names */
 {				/* (requires well-defined enum SIR_TypeType) */
 
 "bool",				/* SIR_TYPE_BOOL */
@@ -100,11 +104,11 @@ NULL				/* SIR_TYPE_INTERFACE *//* set dynamically */
 
 SIR_Type::SIR_Type(		/* constructor #1 (for built-in types) */
 	SIR_TYPETYPE	Type,
-	bool		Const = false,
-	bool		Volatile = false,
-	int		LeftBound = 0,
-	int		RightBound = 0,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	bool		Const /* = false */,
+	bool		Volatile /* = false */,
+	int		LeftBound /* = 0 */,
+	int		RightBound /* = 0 */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 
 SIR_Type::Type = Type;
@@ -188,11 +192,11 @@ SIR_Type::Alias		= NULL;
 SIR_Type::SIR_Type(		/* constructor #2 (for composite types) */
 	sir_type	*SubType,	/* might be NULL (for abstract types) */
 	SIR_TYPETYPE	Type,
-	sir_type_ptrs	*Parameters = NULL,
-	int		Size = 0,
-	bool		Const = false,
-	bool		Volatile = false,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	sir_type_ptrs	*Parameters /* = NULL */,
+	int		Size /* = 0 */,
+	bool		Const /* = false */,
+	bool		Volatile /* = false */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 
 SIR_Type::Type = Type;
@@ -261,8 +265,8 @@ SIR_Type::Alias		= NULL;
 SIR_Type::SIR_Type(		/* constructor #3 (for class types) */
 	SIR_TYPETYPE	Type,
 	const char	*NamePtr,
-	sir_symbol	*ClassSymbol = NULL,
-	sir_type_ptrs	*Parameters = NULL)
+	sir_symbol	*ClassSymbol /* = NULL */,
+	sir_type_ptrs	*Parameters /* = NULL */)
 {
 
 assert(  (Type == SIR_TYPE_BEHAVIOR)
@@ -316,9 +320,9 @@ SIR_Type::SIR_Type(		/* constructor #5 (for user-defined types) */
 	SIR_TYPETYPE	Type,
 	const char	*NamePtr,
 	sir_user_type	*UserType,
-	bool		Const = false,
-	bool		Volatile = false,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	bool		Const /* = false */,
+	bool		Volatile /* = false */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 
 assert(  (Type == SIR_TYPE_STRUCT)
@@ -633,12 +637,12 @@ return(ConstType(Type));
 
 
 ERROR SIR_Type::Check(		/* performs general legal checks on this type */
-	bool		VoidIsOK = false,
-	bool		IsPort = false,
-	bool		IsExternOrTypedef = false,
-	bool		IsPointer = false,	/* used in recursion */
-	bool		IsFunction = false,	/* used in recursion */
-	bool		IsArray = false)	/* used in recursion */
+	bool		VoidIsOK /* = false */,
+	bool		IsPort /* = false */,
+	bool		IsExternOrTypedef /* = false */,
+	bool		IsPointer /* = false */,	/* used in recursion */
+	bool		IsFunction /* = false */,	/* used in recursion */
+	bool		IsArray /* = false */)		/* used in recursion */
 {
 string		TmpErrMsg;
 
@@ -1035,9 +1039,9 @@ return((sir_types*) Head());
 
 
 sir_type *SIR_Type::Modified(	/* returns a (probably new) modified type */
-	bool		Const = false,	/* (must be in a table) */
-	bool		Volatile = false,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	bool		Const /* = false */,	/* (must be in a table) */
+	bool		Volatile /* = false */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 
 assert(GetTable() != NULL);
@@ -1049,8 +1053,8 @@ return(GetTable()->FindOrInsertModifiedType(this,
 
 
 unsigned int SIR_Type::SizeOf(		/* compute the size of this type */
-	unsigned int	*Alignment = NULL,	/* obtain alignment (NULL) */
-	BOOL		SpecCsize = TRUE)	/* SpecC or C++ size */
+	unsigned int	*Alignment /* = NULL */, /* obtain alignment (NULL) */
+	BOOL		SpecCsize /* = TRUE */)	 /* SpecC or C++ size */
 {
 sir_member	*Member;
 unsigned int	Size,
@@ -1278,7 +1282,7 @@ return(0);	/* should never be executed */
 
 const char *SIR_Type::PrintDirection(	/* prints a SpecC port direction */
 	SIR_DIRECTION	Direction,	/* (eg. "out" or "out ") */
-	BOOL		AppendSpace = false)
+	BOOL		AppendSpace /* = false */)
 {
 
 switch(Direction)
@@ -1319,18 +1323,60 @@ return(NULL);	/* never reached */
 } /* end of SIR_Type::PrintDirection */
 
 
-const char *SIR_Type::PrettyString(	/* prints a SpecC type */
-	string		*Buffer,	/* (Buffer must be initialized */
-	BOOL	IncludeNotes = FALSE,	/* with SymbolName or "" for   */
-	BOOL	CplusplusMode = FALSE,	/* abstract types)             */
-	BOOL	UserTypeDefReq = FALSE,
-	BOOL	UseShadowName = FALSE)
+const char *SIR_Type::PrettyString(	/* prints a SpecC type (#1) */
+	string	*Buffer,		/* (Buffer must be initialized */
+	BOOL	IncludeNotes /* = FALSE */,	/* with SymbolName or "" for */
+	BOOL	CplusplusMode /* = FALSE */,	/* abstract types)           */
+	BOOL	UserTypeDefReq /* = FALSE */,
+	BOOL	UseShadowName /* = FALSE */)
+{
+string		Prefix,
+		Postfix;
+int		LastChar;
+
+assert(Buffer != NULL);
+
+Prefix = "";
+Postfix = "";
+
+PrettyString(&Prefix, &Postfix,
+		IncludeNotes, CplusplusMode, UserTypeDefReq, UseShadowName);
+
+if (  (Buffer->empty())
+    &&(Postfix.empty()))
+   { assert(Prefix.length() >= 1);
+     LastChar = Prefix.length()-1;
+     if (Prefix.chars()[LastChar] == ' ')	/* optimize away  */
+	{ *Buffer = Prefix.before(LastChar);	/* trailing space */
+	 } /* fi */
+     else
+	{ *Buffer = Prefix;
+	 } /* esle */
+    } /* fi */
+else
+   { Buffer->prepend(Prefix);
+     *Buffer += Postfix;
+    } /* esle */
+
+return(Buffer->chars());
+
+} /* end of SIR_Type::PrettyString #1 */
+
+
+void SIR_Type::PrettyString(		/* prints a SpecC type (#2) */
+	string	*PrefixReturn,			/* (both Prefix- and Postfix- */
+	string	*PostfixReturn,			/* Return must be initialized */
+	BOOL	IncludeNotes /* = FALSE */,	/* with an empty string ("")  */
+	BOOL	CplusplusMode /* = FALSE */,
+	BOOL	UserTypeDefReq /* = FALSE */,
+	BOOL	UseShadowName /* = FALSE */)
 {
 sir_type_ptr	*Param;
 string		ParamBuffer,
 		TmpString;
 
-assert(Buffer != NULL);
+assert(PrefixReturn != NULL);
+assert(PostfixReturn != NULL);
 
 switch(Type)
    { case SIR_TYPE_BOOL:
@@ -1342,28 +1388,28 @@ switch(Type)
      case SIR_TYPE_UINT:
      case SIR_TYPE_LONG:
      case SIR_TYPE_ULONG:
-	{ Buffer->prepend(" ");
-	  Buffer->prepend(Name);
+	{ PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(Name);
 	  break;
 	 }
      case SIR_TYPE_LONGLONG:
 	{
 #ifdef HAVE_LLONG
-	  Buffer->prepend(" ");
-	  Buffer->prepend(Name);
+	  PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(Name);
 #else /* !HAVE_LLONG */
 	  if (CplusplusMode)
-	     { Buffer->prepend("<" SIR_LLONG_SIGNED_FLAG "> ");
+	     { PrefixReturn->prepend("<" SIR_LLONG_SIGNED_FLAG "> ");
 	       if (UseShadowName)
-		  { Buffer->prepend(SIR_LLONG_SHADOW_NAME);
+		  { PrefixReturn->prepend(SIR_LLONG_SHADOW_NAME);
 		   } /* fi */
 	       else
-		  { Buffer->prepend(SIR_LLONG_CLASS_NAME);
+		  { PrefixReturn->prepend(SIR_LLONG_CLASS_NAME);
 		   } /* esle */
 	      } /* fi */
 	  else
-	     { Buffer->prepend(" ");
-	       Buffer->prepend(Name);
+	     { PrefixReturn->prepend(" ");
+	       PrefixReturn->prepend(Name);
 	      } /* esle */
 #endif /* HAVE_LLONG */
 	  break;
@@ -1371,21 +1417,21 @@ switch(Type)
      case SIR_TYPE_ULONGLONG:
 	{
 #ifdef HAVE_LLONG
-	  Buffer->prepend(" ");
-	  Buffer->prepend(Name);
+	  PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(Name);
 #else /* !HAVE_LLONG */
 	  if (CplusplusMode)
-	     { Buffer->prepend("<" SIR_LLONG_UNSIGNED_FLAG "> ");
+	     { PrefixReturn->prepend("<" SIR_LLONG_UNSIGNED_FLAG "> ");
 	       if (UseShadowName)
-		  { Buffer->prepend(SIR_LLONG_SHADOW_NAME);
+		  { PrefixReturn->prepend(SIR_LLONG_SHADOW_NAME);
 		   } /* fi */
 	       else
-		  { Buffer->prepend(SIR_LLONG_CLASS_NAME);
+		  { PrefixReturn->prepend(SIR_LLONG_CLASS_NAME);
 		   } /* esle */
 	      } /* fi */
 	  else
-	     { Buffer->prepend(" ");
-	       Buffer->prepend(Name);
+	     { PrefixReturn->prepend(" ");
+	       PrefixReturn->prepend(Name);
 	      } /* esle */
 #endif /* HAVE_LLONG */
 	  break;
@@ -1393,8 +1439,8 @@ switch(Type)
      case SIR_TYPE_FLOAT:
      case SIR_TYPE_DOUBLE:
      case SIR_TYPE_LONGDOUBLE:
-	{ Buffer->prepend(" ");
-	  Buffer->prepend(Name);
+	{ PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(Name);
 	  break;
 	 }
      case SIR_TYPE_BIT:
@@ -1409,7 +1455,7 @@ switch(Type)
 	     { TmpString.form("%s[%d:%d] ",
 					Name, LeftBound, RightBound);
 	      } /* esle */
-	  Buffer->prepend(TmpString);
+	  PrefixReturn->prepend(TmpString);
 	  break;
 	 }
      case SIR_TYPE_UBIT:
@@ -1424,48 +1470,49 @@ switch(Type)
 	     { TmpString.form("%s[%d:%d] ",
 					Name, LeftBound, RightBound);
 	      } /* esle */
-	  Buffer->prepend(TmpString);
+	  PrefixReturn->prepend(TmpString);
 	  break;
 	 }
      case SIR_TYPE_VOID:
-	{ Buffer->prepend(" ");
-	  Buffer->prepend(Name);
+	{ PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(Name);
 	  break;
 	 }
      case SIR_TYPE_EVENT:
-	{ Buffer->prepend(" ");
+	{ PrefixReturn->prepend(" ");
 	  if (CplusplusMode)
-	     { Buffer->prepend(SIR_SIM_EVENT_NAME);
+	     { PrefixReturn->prepend(SIR_SIM_EVENT_NAME);
 	      } /* fi */
 	  else
-	     { Buffer->prepend(Name);
+	     { PrefixReturn->prepend(Name);
 	      } /* esle */
 	  break;
 	 }
      case SIR_TYPE_POINTER:
 	{ if (Volatile)
-	     { Buffer->prepend("volatile ");
+	     { PrefixReturn->prepend("volatile ");
 	      } /* fi */
 	  if (Const)
-	     { Buffer->prepend("const ");
+	     { PrefixReturn->prepend("const ");
 	      } /* fi */
-	  Buffer->prepend(Name);
+	  PrefixReturn->prepend(Name);
 	  if (  (SubType->Type == SIR_TYPE_FUNCTION) /* pointer to function? */
 	      ||(SubType->Type == SIR_TYPE_ARRAY))   /* pointer to array? */
-	     { Buffer->prepend("(");
-	       *Buffer += ")";
+	     { PrefixReturn->prepend("(");
+	       *PostfixReturn += ")";
 	      } /* fi */
-	  SubType->PrettyString(Buffer, IncludeNotes, CplusplusMode);
+	  SubType->PrettyString(PrefixReturn, PostfixReturn,
+				IncludeNotes, CplusplusMode);
 	  if (! CplusplusMode)
-	     { Buffer->prepend(PrintDirection(Direction, true));
+	     { PrefixReturn->prepend(PrintDirection(Direction, true));
 	      } /* fi */
-	  return(Buffer->chars());
+	  return;
 	 }
      case SIR_TYPE_STRUCT:
      case SIR_TYPE_UNION:
      case SIR_TYPE_ENUM:
-	{ Buffer->prepend(" ");
-	  Buffer->prepend(UserType->PrettyString(
+	{ PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(UserType->PrettyString(
 				&ParamBuffer, IncludeNotes,
 				CplusplusMode, UserTypeDefReq));
 	  break;
@@ -1473,16 +1520,17 @@ switch(Type)
      case SIR_TYPE_ARRAY:
 	{ if (Size != SIR_UNKNOWN_ARRAY_SIZE)
 	     { TmpString.form("[%d]", Size);
-	       *Buffer += TmpString;
+	       *PostfixReturn += TmpString;
 	      } /* fi */
 	  else
-	     { *Buffer += "[]";
+	     { *PostfixReturn += "[]";
 	      } /* esle */
-	  SubType->PrettyString(Buffer, IncludeNotes, CplusplusMode);
+	  SubType->PrettyString(PrefixReturn, PostfixReturn,
+				IncludeNotes, CplusplusMode);
 	  break;
 	 }
      case SIR_TYPE_FUNCTION:
-	{ *Buffer += "(";
+	{ *PostfixReturn += "(";
 	  Param = Parameters->First();
 	  while(Param)
 	     { if (  (CplusplusMode)
@@ -1492,20 +1540,22 @@ switch(Type)
 	       else
 		  { ParamBuffer = "";
 		   } /* esle */
-	       *Buffer += Param->Type->PrettyString(&ParamBuffer, IncludeNotes,
-							CplusplusMode);
+	       *PostfixReturn += Param->Type->PrettyString(&ParamBuffer,
+						IncludeNotes, CplusplusMode);
 	       Param = Param->Succ();
 	       if (Param)
-		  { *Buffer += ", ";
+		  { *PostfixReturn += ", ";
 		   } /* fi */
 	      } /* elihw */
-	  *Buffer += ")";
-	  SubType->PrettyString(Buffer, IncludeNotes, CplusplusMode);
+	  *PostfixReturn += ")";
+	  SubType->PrettyString(PrefixReturn, PostfixReturn,
+				IncludeNotes, CplusplusMode);
 	  break;
 	 }
      case SIR_TYPE_ANY_TYPE:
-	{ assert(Buffer->empty());
-	  *Buffer = Name;
+	{ assert(PrefixReturn->empty());
+	  assert(PostfixReturn->empty());
+	  *PrefixReturn = Name;
 	  assert(! Volatile);
 	  assert(! Const);
 	  assert(Direction == SIR_PORT_NONE);
@@ -1514,8 +1564,8 @@ switch(Type)
      case SIR_TYPE_BEHAVIOR:
      case SIR_TYPE_CHANNEL:
      case SIR_TYPE_INTERFACE:
-	{ Buffer->prepend(" ");
-	  Buffer->prepend(Name);
+	{ PrefixReturn->prepend(" ");
+	  PrefixReturn->prepend(Name);
 	  assert(! Volatile);
 	  assert(! Const);
 	  assert(Direction == SIR_PORT_NONE);
@@ -1527,18 +1577,16 @@ switch(Type)
     } /* hctiws */
 
 if (Volatile)
-   { Buffer->prepend("volatile ");
+   { PrefixReturn->prepend("volatile ");
     } /* fi */
 if (Const)
-   { Buffer->prepend("const ");
+   { PrefixReturn->prepend("const ");
     } /* fi */
 if (! CplusplusMode)
-   { Buffer->prepend(PrintDirection(Direction, true));
+   { PrefixReturn->prepend(PrintDirection(Direction, true));
     } /* fi */
 
-return(Buffer->chars());
-
-} /* end of SIR_Type::PrettyString */
+} /* end of SIR_Type::PrettyString #2 */
 
 
 void SIR_Type::MarkUsed(void)		/* mark this type as being used */
@@ -1798,8 +1846,8 @@ fprintf(stderr, "GARBAGE_COLLECTOR: Ending with %d type entries...\n",
 
 sir_type *SIR_Types::FindOrInsert(	/* find an entry or insert it */
 	SIR_CONSTTYPE	ConstType,	/* if it not exists (#1)      */
-	int		LeftBound = 0,
-	int		RightBound = 0)
+	int		LeftBound /* = 0 */,
+	int		RightBound /* = 0 */)
 {
 SIR_TYPETYPE	MyType;
 sir_type	*TypeEntry;
@@ -1847,11 +1895,11 @@ return(NULL);	/* never reached */
 
 sir_type *SIR_Types::FindOrInsert(	/* find an entry or insert it */
 	SIR_TYPETYPE	Type,		/* if it not exists (#2)      */
-	bool		Const = false,
-	bool		Volatile = false,
-	int		LeftBound = 0,
-	int		RightBound = 0,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	bool		Const /* = false */,
+	bool		Volatile /* = false */,
+	int		LeftBound /* = 0 */,
+	int		RightBound /* = 0 */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 
 return(FindOrInsert(
@@ -1927,11 +1975,11 @@ return(FindOrInsert(
 sir_type *SIR_Types::FindOrInsert(	/* find an entry or insert it */
 	SIR_TYPETYPE	Type,		/* if it not exists (#4)      */
 	sir_type	*SubType,	/* (for composite types)      */
-	sir_type_ptrs	*Parameters = NULL,
-	int		Size = 0,
-	bool		Const = false,
-	bool		Volatile = false,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	sir_type_ptrs	*Parameters /* = NULL */,
+	int		Size /* = 0 */,
+	bool		Const /* = false */,
+	bool		Volatile /* = false */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 
 return(FindOrInsert(
@@ -1979,9 +2027,9 @@ else
 
 sir_type *SIR_Types::FindOrInsertModifiedType( /* find/insert a modified type */
 	sir_type	*OrigType,
-	bool		Const = false,
-	bool		Volatile = false,
-	SIR_DIRECTION	Direction = SIR_PORT_NONE)
+	bool		Const /* = false */,
+	bool		Volatile /* = false */,
+	SIR_DIRECTION	Direction /* = SIR_PORT_NONE */)
 {
 sir_type	*NewType;
 
@@ -2030,7 +2078,7 @@ sir_type *SIR_Types::FindOrInsertClassType( /* find or insert a class type */
 	SIR_TYPETYPE	Type,
 	const char	*Name,
 	sir_type_ptrs	*Parameters,
-	sir_symbol	*ClassSymbol = NULL)
+	sir_symbol	*ClassSymbol /* = NULL */)
 {
 
 assert(  (Type == SIR_TYPE_BEHAVIOR)
@@ -2087,7 +2135,7 @@ else
 int SIR_Types::TypeCmp(		/* type comparision (like strcmp) */
 	sir_type	*Type1,	/* (recursive for complex types!) */
 	sir_type	*Type2,
-	bool		IgnoreDirection = false)
+	bool		IgnoreDirection /* = false */)
 {
 int		CmpVal;
 
@@ -2243,7 +2291,7 @@ SIR_TypePtr::~SIR_TypePtr(void)		/* destructor */
 
 
 SIR_TypePtrs::SIR_TypePtrs(		/* constructor #1 */
-	sir_type_ptr	*FirstEntry = NULL) :
+	sir_type_ptr	*FirstEntry /* = NULL */) :
 		SIR_List<SIR_TypePtr>(FirstEntry)
 {
 
