@@ -2,6 +2,8 @@
 // bit.cc: Bit Vector Library
 // ============================================================================
 //
+// 09/27/01 <gerstl>	Fixed bug in ubit2str() with base != 2
+// 08/10/01 <gerstl>	Introduced len_t, bnd_t; changed shift cnt to unsigned
 // 06/01/01 <gerstl>	Resolved conflict of method names with ANSI (keywords?)
 // 05/24/01 <gerstl>	Initial version
 //
@@ -98,12 +100,12 @@ _bitbus_element* _bitbus_element::concat(_bitbus_element* list)
 
 
 // Return newly allocated list that is a slice of this list.
-_bitbus_element* _bitbus_element::slice(int l, int r)
+_bitbus_element* _bitbus_element::slice(len_t l, len_t r)
 {
-  int i, newsl, newsr;
-  int ellen = _BITLEN(sl, sr);
-  int max = _MAXLEN(l,r);
-  int min = _MINLEN(l,r);
+  len_t i, newsl, newsr;
+  len_t ellen = _BITLEN(sl, sr);
+  len_t max = _MAXLEN(l,r);
+  len_t min = _MINLEN(l,r);
   _bitbus_element* head = NULL;
   _bitbus_element** tmp = &head;
   _bitbus_element* el;
@@ -150,7 +152,7 @@ _bitbus_element* _bitbus_element::slice(int l, int r)
 // Sync with original vectors
 void _bitbus_element::put(const _bit& buf)
 {
-  int i, len;
+  len_t i, len;
   _bitbus_element *el = this;
   
   for (i = 0; el; i += len, el = el->next) {
@@ -163,7 +165,7 @@ void _bitbus_element::get(const _bit& buf)
 {
   // ## We could do some checksumming here to find out if the originals
   //    really have changed (room for optimization)
-  int i, len;
+  len_t i, len;
   const _bitbus_element *el = this;
   
   for(i = 0; el; i += len, el = el->next) {
@@ -179,9 +181,9 @@ void _bitbus_element::get(const _bit& buf)
 
 /* --- Shifting --- */
 
-_bit& _bit::shlOp(const _bit& op1, int sh, bool mask)
+_bit& _bit::shlOp(const _bit& op1, unsigned int sh, bool mask)
 {
-  ASSERT((op1.len+sh) >= len);
+  ASSERT((len_t)(op1.len+sh) >= len);
   
   // Divide shift in chunks
   int c = (sh / _BITCHUNK_SIZE);
@@ -214,7 +216,7 @@ _bit& _bit::shlOp(const _bit& op1, int sh, bool mask)
 }
 
 
-_bit& _bit::shrOp(const _bit& op1, int sh)
+_bit& _bit::shrOp(const _bit& op1, unsigned int sh)
 {
   int i;
   
@@ -637,7 +639,7 @@ _bit& _bit::divOp(const _bit& n, const _bit& d, _bit* r)
 _bit& _bit::set (const char* str, bool usign, unsigned int base)
 {
   unsigned int digit;
-  int  l   = 0;
+  len_t  l = 0;
   bool neg = false;
 
   // Eat leading whitespace
@@ -720,7 +722,7 @@ _bit& _bit::set (const char* str, bool usign, unsigned int base)
 
 /* --- Assignment of bit slice --- */
 
-void _bit::fill(int l, int r, int val)
+void _bit::fill(len_t l, len_t r, int val)
 {
   ASSERT( l >= r );
   ASSERT( _BITCHUNK(l) < _BITCHUNKS(len) );
@@ -764,16 +766,15 @@ void _bit::fill(int l, int r, int val)
 }
 
 
-void _bit::copyBits( const _bit& src, int sl, int sr, int dl, int dr )
+void _bit::copyBits( const _bit& src, len_t sl, len_t sr, len_t dl, len_t dr )
 {
-  int i;
   _bit::chunk* srcvec, *tmp = 0;
   
   ASSERT(dl < len);
   ASSERT(dr < len);
 
-  int slen = _BITLEN(sl,sr);
-  int dlen = _BITLEN(dl,dr);
+  len_t slen = _BITLEN(sl,sr);
+  len_t dlen = _BITLEN(dl,dr);
 
   // Make a copy of source if source and destination are the same
   if( src.addr() == addr() )
@@ -815,7 +816,7 @@ void _bit::copyBits( const _bit& src, int sl, int sr, int dl, int dr )
   }
   else if( dlen > slen )
   {
-    int rfill, lfill = dl;
+    len_t rfill, lfill = dl;
     
     // Shrink destination
     dl -= (dl > dr)? (dlen - slen) : (slen - dlen);
@@ -839,8 +840,9 @@ void _bit::copyBits( const _bit& src, int sl, int sr, int dl, int dr )
   
   // Make the destination canonical
   if (dl < dr) {
-    i = dr; dr = dl; dl = i;
-    i = sr; sr = sl; sl = i;
+    len_t t;
+    t = dr; dr = dl; dl = t;
+    t = sr; sr = sl; sl = t;
   }
 
   int schnk = sr / _BITCHUNK_SIZE;
@@ -917,7 +919,7 @@ void _bit::copyBits( const _bit& src, int sl, int sr, int dl, int dr )
     unsigned sptr = 0x01 << ssh;
     unsigned dptr = 0x01 << dsh;
 
-    for (i = 0; i <= (dl - dr); i++) 
+    for (len_t i = 0; i <= (dl - dr); i++) 
     {
       // Set new bit according to source bit
       if (srcvec[schnk].uval & sptr) 
@@ -1007,7 +1009,7 @@ double _bit::toDouble(void) const
 char *ubit2str(unsigned int base, char *endptr, const _bit& b)
 {
   unsigned int digit;
-  int l = b.length();
+  _bit::len_t l = b.length();
 
   _bit ub(l, true);   // create local copy, unsigned!
   ub = b;
@@ -1021,21 +1023,21 @@ char *ubit2str(unsigned int base, char *endptr, const _bit& b)
                ub >>= 1;
                l--;
                break;
-      case  4: digit = ub.toUInt() & 0x11;
+      case  4: digit = ub.toUInt() & 0x3;
                ub >>= 2;
-               l -= 2;
+               if(l > 2) l -= 2; else l = 0;
                break;
-      case  8: digit = ub.toUInt() & 0x111;
+      case  8: digit = ub.toUInt() & 0x7;
                ub >>= 3;
-               l -= 3;
+               if(l > 3) l -= 3; else l = 0;
                break;
-      case 16: digit = ub.toUInt() & 0x1111;
+      case 16: digit = ub.toUInt() & 0xF;
                ub >>= 4;
-               l -= 4;
+               if(l > 4) l -= 4; else l = 0;
                break;
-      case 32: digit = ub.toUInt() & 0x11111;
+      case 32: digit = ub.toUInt() & 0x1F;
                ub >>= 5;
-               l -= 5;
+               if(l > 5) l -= 5; else l = 0;
                break;
       default: digit = (ub % base).toUInt();
                ub /= base;
@@ -1464,12 +1466,6 @@ int errSignExt(void)
 }
 
 
-int errAddrOp(void)
-{
-  printf("Error: Address operator doesn't point to first bit vector element!!\n");
-  return 4;
-}
-
 int errSizeOp(void)
 {
   printf("Error: Calculation of bit class size doesn't match sizeof() result!!\n");
@@ -1484,6 +1480,7 @@ int err(const char * s)
 
 int main(void)
 {
+  char* p;
   char buf[128], buf1[64], buf2[64];
   _bit_<71>   a;
   bit<6,true> b = "111111";
@@ -1503,15 +1500,31 @@ int main(void)
   if (a.toLLong() != -LONG_LONG(1)) return errSignExt();
 #endif  
   
-  // Test for correct address operator
-  // Address of bit vector should point to first integer chunk of data
-  // (and not, for example, to the virtual method table pointer...)
-  if ( ( ((void*)(&a)) != ((void*)(((_bit)a).addr())) ) ||
-       ( ((void*)(&b)) != ((void*)(((_bit)b).addr())) ) ) return errAddrOp();
-
   // Test for correct size calculations
   if ( (sizeof(a) != _SIZEOFBITCLASS(71)) || (sizeof(b) != _SIZEOFBITCLASS(6)) )
     return errSizeOp();
+
+  // Test output routines
+  if(strcmp(p = ubit2str(2, &buf1[63], b), "111111") != 0) {
+    sprintf(buf, "ubit2str(2, '111111') returns %s\n", p);
+    return err(buf);
+  }
+  if(strcmp(p = ubit2str(4, &buf1[63], b), "333") != 0) {
+    sprintf(buf, "ubit2str(4, '333') returns %s\n", p);
+    return err(buf);
+  }
+  if(strcmp(p = ubit2str(8, &buf1[63], b), "77") != 0) {
+    sprintf(buf, "ubit2str(8, '077') returns %s\n", p);
+    return err(buf);
+  }
+  if(strcmp(p = ubit2str(16, &buf1[63], b), "3F") != 0) {
+    sprintf(buf, "ubit2str(16, '0x3f') returns %s\n", p);
+    return err(buf);
+  }
+  if(strcmp(p = ubit2str(32, &buf1[63], b), "1V") != 0) {
+    sprintf(buf, "ubit2str(32, '1V') returns %s\n", p);
+    return err(buf);
+  }
   
   printf("Doing a test of miscellaneous bit operations...\n");
   if( ! testMisc(a, b) ) return err("Error in calculation");
