@@ -2,7 +2,7 @@
 // longlong.h: "long long" data type support
 // ============================================================================
 // 
-//
+// 11/28/01 <gerstl>	Removed ambiguities, move more casting into scc
 // 05/24/01 <gerstl>	Initial version
 //
 // ----------------------------------------------------------------------------
@@ -187,7 +187,6 @@ public:
   __longlong()                                                             { }
   __longlong(unsigned long hi, unsigned long lo)         { u1 = hi; u0 = lo; }
   __longlong(const char* s, unsigned int base = 0)	     { set(s, base); }
-  __longlong(const __longlong& i)                               { *this = i; }
   __longlong(int i)                                             { *this = i; }
   __longlong(unsigned int i)                                    { *this = i; }
   __longlong(long i)                                            { *this = i; }
@@ -197,7 +196,6 @@ public:
   
   // Assignement
   __longlong& operator=(const char* s)       { set(s);                   return *this; }
-  __longlong& operator=(const __longlong& i) { u0 = i.u0; u1 = i.u1;     return *this; }
   __longlong& operator=(int i)               { s0 = i; s1 = i<0? -1 : 0; return *this; }
   __longlong& operator=(unsigned int i)      { u0 = i; u1 = 0;           return *this; }
   __longlong& operator=(long i)              { s0 = i; s1 = i<0? -1 : 0; return *this; }
@@ -227,10 +225,10 @@ public:
   // Comparison
   friend bool operator==(const __longlong&, const __longlong&);
   friend bool operator!=(const __longlong&, const __longlong&);
-  friend bool operator> (const __longlong&, const __longlong&);
-  friend bool operator>=(const __longlong&, const __longlong&);
-  friend bool operator< (const __longlong&, const __longlong&);
-  friend bool operator<=(const __longlong&, const __longlong&);
+  friend bool operator> (const _longlong<true>&, const _longlong<true>&);
+  friend bool operator>=(const _longlong<true>&, const _longlong<true>&);
+  friend bool operator< (const _longlong<true>&, const _longlong<true>&);
+  friend bool operator<=(const _longlong<true>&, const _longlong<true>&);
   friend bool operator> (const _longlong<false>&, const _longlong<false>&);
   friend bool operator>=(const _longlong<false>&, const _longlong<false>&);
   friend bool operator< (const _longlong<false>&, const _longlong<false>&);
@@ -267,7 +265,6 @@ public:
   _longlong(unsigned long hi, unsigned long lo)  : __longlong(hi, lo)   { }
   _longlong(const char* s, unsigned int base = 0): __longlong(s, base)  { }
   _longlong(const __longlong& ll)                : __longlong(ll)       { }
-  _longlong(const _longlong_<usign>& ll)         : __longlong(ll)       { }
   _longlong(int i)                               : __longlong(i)        { }
   _longlong(unsigned int i)                      : __longlong(i)        { }
   _longlong(long i)                              : __longlong(i)        { }
@@ -275,10 +272,10 @@ public:
   _longlong(double d, bool usgn)                 : __longlong(d, usgn)  { }
   _longlong(long double d, bool usgn)            : __longlong(d, usgn)  { }
   
-  // Type-true assignements
+  // Type-true assignments
   _longlong& operator=(const char* s)         { return (_longlong&)__longlong::operator=(s);  }
   _longlong& operator=(const __longlong& ll)  { return (_longlong&)__longlong::operator=(ll); }
-  _longlong& operator=(const _longlong_<usign>& ll)
+  _longlong& operator=(const _longlong_<usign>& ll)  
                                               { return (_longlong&)__longlong::operator=(ll); }
   _longlong& operator=(int i)                 { return (_longlong&)__longlong::operator=(i);  }
   _longlong& operator=(unsigned int i)        { return (_longlong&)__longlong::operator=(i);  }
@@ -291,12 +288,12 @@ public:
   _longlong& operator+=(const __longlong& ll) { return (_longlong&)__longlong::operator+=(ll); }
   _longlong& operator-=(const __longlong& ll) { return (_longlong&)__longlong::operator-=(ll); }
   _longlong& operator*=(const __longlong& ll) { return (_longlong&)__longlong::operator*=(ll); }
-  inline _longlong& operator/=(const _longlong& ll);
-  inline _longlong& operator%=(const _longlong& ll);
-  template<bool usign2>
-    _longlong& operator/=(const _longlong<usign2>& ll) { return (_longlong&)__longlong::operator/=(ll); }
-  template<bool usign2>
-    _longlong& operator%=(const _longlong<usign2>& ll) { return (_longlong&)__longlong::operator%=(ll); }
+  inline _longlong& operator/=(const __longlong& ll);
+  inline _longlong& operator%=(const __longlong& ll);
+  template<bool us>
+    inline _longlong& operator/=(const _longlong<us>& ll);
+  template<bool us>
+    inline _longlong& operator%=(const _longlong<us>& ll);
   inline const _longlong  operator-() const;
   inline const _longlong& operator+() const; 
   
@@ -343,19 +340,19 @@ public:
   _longlong<usign>& operator=(long double d)        { return (_longlong<usign>&)(*this) = d; }
 
   // arithmetic
-  _longlong<usign>& operator+=(const __longlong& ll)       { return (_longlong<usign>&)(*this) += ll; }
-  _longlong<usign>& operator-=(const __longlong& ll)       { return (_longlong<usign>&)(*this) -= ll; }
-  _longlong<usign>& operator*=(const __longlong& ll)       { return (_longlong<usign>&)(*this) *= ll; }
-  _longlong<usign>& operator/=(const _longlong<usign>& ll) { return (_longlong<usign>&)(*this) /= ll; }
-  _longlong<usign>& operator%=(const _longlong<usign>& ll) { return (_longlong<usign>&)(*this) %= ll; }
-  template<bool usign2>
-    _longlong<usign>& operator/=(const _longlong<usign2>& ll)  { return (_longlong<usign>&)(*this) /= ll; }
-  template<bool usign2>
-    _longlong<usign>& operator%=(const _longlong<usign2>& ll)  { return (_longlong<usign>&)(*this) %= ll; }
-  template<bool usign2>
-    _longlong<usign>& operator/=(const _longlong_<usign2>& ll) { return (_longlong<usign>&)(*this) /= (_longlong<usign2>&)ll; }
-  template<bool usign2>
-    _longlong<usign>& operator%=(const _longlong_<usign2>& ll) { return (_longlong<usign>&)(*this) %= (_longlong<usign2>&)ll; }
+  _longlong<usign>& operator+=(const __longlong& ll) { return (_longlong<usign>&)(*this) += ll; }
+  _longlong<usign>& operator-=(const __longlong& ll) { return (_longlong<usign>&)(*this) -= ll; }
+  _longlong<usign>& operator*=(const __longlong& ll) { return (_longlong<usign>&)(*this) *= ll; }
+  _longlong<usign>& operator/=(const __longlong& ll) { return (_longlong<usign>&)(*this) /= ll; }
+  _longlong<usign>& operator%=(const __longlong& ll) { return (_longlong<usign>&)(*this) %= ll; }
+  template<bool us>
+    _longlong<usign>& operator/=(const _longlong<us>& ll)  { return (_longlong<usign>&)(*this) /= ll; }
+  template<bool us>
+    _longlong<usign>& operator%=(const _longlong<us>& ll)  { return (_longlong<usign>&)(*this) %= ll; }
+  template<bool us>
+    _longlong<usign>& operator/=(const _longlong_<us>& ll) { return (_longlong<usign>&)(*this) /= (_longlong<us>&)ll; }
+  template<bool us>
+    _longlong<usign>& operator%=(const _longlong_<us>& ll) { return (_longlong<usign>&)(*this) %= (_longlong<us>&)ll; }
   
   _longlong<usign>  operator-() const { return -((_longlong<usign>&)(*this)); }
   _longlong<usign>& operator+() const { return +((_longlong<usign>&)(*this)); }
@@ -392,7 +389,7 @@ inline _longlong<false> operator+( const _longlong<false>& ll1, const _longlong<
 }
   
 // unsigned
-inline _longlong<true> operator+( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator+( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll += ll2;
 }
@@ -407,7 +404,7 @@ inline _longlong<false> operator-( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator-( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator-( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll -= ll2;
 }
@@ -422,7 +419,7 @@ inline _longlong<false> operator*( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator*( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator*( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll *= ll2;
 }
@@ -437,7 +434,7 @@ inline _longlong<false> operator/( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator/( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator/( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll /= ll2;
 }
@@ -452,7 +449,7 @@ inline _longlong<false> operator%( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator%( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator%( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll %= ll2;
 }
@@ -467,7 +464,7 @@ inline _longlong<false> operator&( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator&( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator&( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll &= ll2;
 }
@@ -482,7 +479,7 @@ inline _longlong<false> operator|( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator|( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator|( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll |= ll2;
 }
@@ -497,7 +494,7 @@ inline _longlong<false> operator^( const _longlong<false>& ll1, const _longlong<
 }
 
 // unsigned
-inline _longlong<true> operator^( const __longlong& ll1, const __longlong& ll2 ) {
+inline _longlong<true> operator^( const _longlong<true>& ll1, const _longlong<true>& ll2 ) {
   _longlong<true> ll( ll1 );
   return ll ^= ll2;
 }
@@ -565,25 +562,25 @@ inline bool operator<=( const _longlong<false>& a, const _longlong<false>& b ) {
 }
 
 // unsigned
-inline bool operator>( const __longlong& a, const __longlong& b ) {
+inline bool operator>( const _longlong<true>& a, const _longlong<true>& b ) {
   if( a.u1 > b.u1 ) return true;
   if( a.u1 < b.u1 ) return false;
   return (a.u0 > b.u0);
 }
 
-inline bool operator>=( const __longlong& a, const __longlong& b ) {
+inline bool operator>=( const _longlong<true>& a, const _longlong<true>& b ) {
   if( a.u1 > b.u1 ) return true;
   if( a.u1 < b.u1 ) return false;
   return (a.u0 >= b.u0);
 }
 
-inline bool operator<( const __longlong& a, const __longlong& b ) {
+inline bool operator<( const _longlong<true>& a, const _longlong<true>& b ) {
   if( a.u1 < b.u1 ) return true;
   if( a.u1 > b.u1 ) return false;
   return (a.u0 < b.u0);
 }
 
-inline bool operator<=( const __longlong& a, const __longlong& b ) {
+inline bool operator<=( const _longlong<true>& a, const _longlong<true>& b ) {
   if( a.u1 < b.u1 ) return true;
   if( a.u1 > b.u1) return false;
   return (a.u0 <= b.u0);
@@ -633,17 +630,19 @@ inline __longlong& __longlong::operator%=(const __longlong& ll) {
 
 // Type-true division
 template<bool usign>
-  inline _longlong<usign>& _longlong<usign>::operator/=(const _longlong<usign>& ll)
-{
+  inline _longlong<usign>& _longlong<usign>::operator/=(const __longlong& ll)
+{    
   // Signed division?
   if( !usign )
   {
+    const _longlong<usign>& op = ll;
+    
     // Make operands positive
     bool nNeg = (s1 < 0);
-    bool dNeg = (ll < 0);
+    bool dNeg = (op < 0);
     if( nNeg ) neg();
     // Divide
-    __longlong::operator/=( dNeg? -ll : ll );
+    __longlong::operator/=( dNeg? -op : op );
     // Adjust result
     if( dNeg ^ nNeg ) neg();
     return *this;
@@ -651,20 +650,31 @@ template<bool usign>
   return (_longlong<usign>&)__longlong::operator/=( ll );
 }
 
+template<bool usign> template<bool us>
+  inline _longlong<usign>& _longlong<usign>::operator/=(const _longlong<us>& ll)
+{
+  if( !usign && !us ) 
+    return operator/=( (__longlong&)ll );
+  else
+    return (_longlong<usign>&)__longlong::operator/=( ll );
+}
+
 
 // Type-true modulo
 template<bool usign>
-  inline _longlong<usign>& _longlong<usign>::operator%=(const _longlong<usign>& ll)
+  inline _longlong<usign>& _longlong<usign>::operator%=(const __longlong& ll)
 {
   // Signed division?
   if( !usign )
   {
+    const _longlong<usign>& op = ll;
+    
     // Make operands positive
     bool nNeg = (s1 < 0);
-    bool dNeg = (ll < 0);
+    bool dNeg = (op < 0);
     if( nNeg ) neg();
     // Divide
-    __longlong::operator%=( dNeg? -ll : ll );
+    __longlong::operator%=( dNeg? -op : op );
     // Adjust result
     if( nNeg ) neg();
     return *this;
@@ -672,6 +682,14 @@ template<bool usign>
   return (_longlong<usign>&)__longlong::operator%=( ll );
 }
 
+template<bool usign> template<bool us>
+  inline _longlong<usign>& _longlong<usign>::operator%=(const _longlong<us>& ll)
+{
+  if( !usign && !us ) 
+    return operator%=( (__longlong&)ll );
+  else
+    return (_longlong<usign>&)__longlong::operator%=( ll );
+}
 
 // Negation
 inline __longlong& __longlong::neg() {
@@ -796,165 +814,6 @@ template<bool usign>
 }
 
 
-/* ------------------------------------------------------------------------- */
-/* Shadow class operations */
-
-
-/* --- Addition --- */
-inline _longlong<false> operator+( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 + (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator+( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    + (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator+( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 + ll2;
-}
-
-/* --- Subtraction --- */
-inline _longlong<false> operator-( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 - (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator-( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    - (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator-( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 - ll2;
-}
-
-/* --- Multiplikation --- */
-inline _longlong<false> operator*( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 * (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator*( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    * (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator*( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 * ll2;
-}
-
-/* --- Division --- */
-inline _longlong<false> operator/( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 / (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator/( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    / (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator/( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 / ll2;
-}
-
-/* --- Modulo --- */
-inline _longlong<false> operator%( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 % (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator%( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    % (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator%( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 % ll2;
-}
-
-/* --- And --- */
-inline _longlong<false> operator&( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 & (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator&( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    & (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator&( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 & ll2;
-}
-
-/* --- Or --- */
-inline _longlong<false> operator|( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 | (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator|( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    | (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator|( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 | ll2;
-}
-
-/* --- Xor --- */
-inline _longlong<false> operator^( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 ^ (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator^( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    ^ (_longlong<false>&)ll2;
-}
-inline _longlong<false> operator^( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 ^ ll2;
-}
-
-/* --- Equal --- */
-inline bool operator==( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 == (_longlong<false>&)ll2;
-}
-inline bool operator==( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    == (_longlong<false>&)ll2;
-}
-inline bool operator==( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 == ll2;
-}
-
-/* --- Unequal --- */
-inline bool operator!=( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 != (_longlong<false>&)ll2;
-}
-inline bool operator!=( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    != (_longlong<false>&)ll2;
-}
-inline bool operator!=( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 != ll2;
-}
-
-/* --- Greater than --- */
-inline bool operator>( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 > (_longlong<false>&)ll2;
-}
-inline bool operator>( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    > (_longlong<false>&)ll2;
-}
-inline bool operator>( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 > ll2;
-}
-
-/* --- Greater or equal --- */
-inline bool operator>=( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 >= (_longlong<false>&)ll2;
-}
-inline bool operator>=( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    >= (_longlong<false>&)ll2;
-}
-inline bool operator>=( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 >= ll2;
-}
-
-/* --- Less than --- */
-inline bool operator<( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 < (_longlong<false>&)ll2;
-}
-inline bool operator<( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    < (_longlong<false>&)ll2;
-}
-inline bool operator<( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 < ll2;
-}
-
-/* --- Less than or equal --- */
-inline bool operator<=( const _longlong_<false>& ll1, const _longlong_<false>& ll2 ) {
-  return (_longlong<false>&)ll1 <= (_longlong<false>&)ll2;
-}
-inline bool operator<=( const _longlong<false>& ll1, const _longlong_<false>& ll2 ) {
-  return ll1                    <= (_longlong<false>&)ll2;
-}
-inline bool operator<=( const _longlong_<false>& ll1, const _longlong<false>& ll2 ) {
-  return (_longlong<false>&)ll1 <= ll2;
-}
-
-
 #endif
 
 #ifdef FIRST_PASS
